### Name: spGGT
### Title: Function for fitting univariate and multivariate Bayesian
###   spatial regression models
### Aliases: spGGT
### Keywords: misc

### ** Examples

## Not run: 
##D 
##D data(FBC07.dat)
##D Y.1 <- FBC07.dat[1:100,"Y.1"]
##D Y.2 <- FBC07.dat[1:100,"Y.2"]
##D coords <- as.matrix(FBC07.dat[1:100,c("coord.X", "coord.Y")])
##D 
##D #############################
##D ##   Univariate models
##D #############################
##D 
##D ##Fit some competing models.
##D ##Full model with nugget (Psi), partial sill (K),
##D ##and spatial range (Phi).
##D K.prior <- prior(dist="IG", shape=2, scale=5)
##D Psi.prior <- prior(dist="IG", shape=2, scale=5)
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D 
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "Psi"=list(starting=5, tuning=0.5, prior=Psi.prior),
##D        "phi"=list(starting=0.1, tuning=0.5, prior=phi.prior)
##D        )
##D 
##D beta.control <- list(update="GIBBS", prior=prior(dist="FLAT"))
##D 
##D run.control <- list("n.samples"=1000, "sp.effects"=TRUE)
##D 
##D Fit.m1 <- spGGT(formula=Y.2~1, run.control=run.control,
##D                  coords=coords, var.update.control=var.update.control,
##D                  beta.update.control=beta.control,
##D                  cov.model="exponential")
##D 
##D plot(Fit.m1$p.samples)
##D summary(Fit.m1$p.samples)
##D 
##D ##First reduced model with no nugget just partial sill and range.
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "phi"=list(starting=0.1, tuning=0.5, prior=phi.prior)
##D        )
##D 
##D Fit.m2 <- spGGT(formula=Y.2~1, run.control=run.control,
##D                  coords=coords, var.update.control=var.update.control,
##D                  beta.update.control=beta.control,
##D                  cov.model="exponential")
##D 
##D plot(Fit.m2$p.samples)
##D 
##D ##Second reduced model with no nugget just partial sill and fixed
##D ##range at the WRONG value, which forces the wrong estimate of K.
##D phi.prior <- prior(dist="FIXED")
##D        
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "phi"=list(starting=0.1, prior=phi.prior)
##D        )
##D 
##D Fit.m3 <- spGGT(formula=Y.2~1, run.control=run.control,
##D                  coords=coords, var.update.control=var.update.control,
##D                  beta.update.control=beta.control,
##D                  cov.model="exponential")
##D plot(Fit.m3$p.samples)
##D 
##D ##For this data, DIC cannot distinguish between the first two
##D ##models but does show that the third model is clearly
##D ##wrong.  An empirical semivariogram is useful
##D ##for recognizing that the full model is the correct choice
##D ##over the second model.
##D print(spDIC(Fit.m1, DIC.unmarg=FALSE))
##D print(spDIC(Fit.m2, DIC.unmarg=FALSE))
##D print(spDIC(Fit.m3, DIC.unmarg=FALSE))
##D 
##D ##Use akima or MBA to produce interpolated surfaces of
##D ##estimated random spatial effects.
##D 
##D m1.surf <- mba.surf(cbind(coords, rowMeans(Fit.m1$sp.effects)),
##D                     no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D image(m1.surf, xaxs="r", yaxs="r",
##D       main="Y.2 random spatial effects")
##D points(coords, pch=19)
##D 
##D ##Now the full model using the Matern spatial correlation function.
##D ##Also, for fun, update the spatial range parameters
##D ##phi and nu in a separate MH block.  This provides finer control
##D ##on the parameters' acceptance rate, but takes a bit longer to
##D ##run.
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D nu.prior <- prior(dist="UNIF", a=0.01, b=2)
##D 
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "Psi"=list(starting=5, tuning=0.5, prior=Psi.prior),
##D        "phi"=list(sample.order=1, starting=0.1,
##D          tuning=0.5, prior=phi.prior),
##D        "nu"=list(sample.order=1, starting=0.1,
##D          tuning=0.5, prior=nu.prior)     
##D        )
##D 
##D run.control <- list("n.samples"=1000, "sp.effects"=FALSE)
##D 
##D Fit.m4 <-
##D   spGGT(formula=Y.2~1, run.control=run.control,
##D          coords=coords, var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="matern")
##D 
##D plot(Fit.m4$p.samples)
##D 
##D ##Solve the Matern correlation function for the estimated
##D ##95% CI for effective range (e.g., distance at which the
##D ##correlation drops to 0.05).
##D phi.hat <-  quantile(Fit.m4$p.samples[,"Phi"], c(0.50,0.025,0.975))
##D nu.hat <- quantile(Fit.m4$p.samples[,"Nu"], c(0.50,0.025,0.975))
##D 
##D matern <- function(cor, eff.d, phi, nu){
##D   cor - (eff.d*phi)^nu/(2^(nu-1)*gamma(nu))*besselK(x=eff.d*phi, nu=nu)
##D }
##D 
##D ##50
##D print(uniroot(matern, interval=c(1, 50),
##D               cor=0.05, phi=phi.hat[1], nu=nu.hat[1])$root)
##D print(uniroot(matern, interval=c(1, 50),
##D               cor=0.05, phi=phi.hat[2], nu=nu.hat[2])$root)
##D print(uniroot(matern, interval=c(1, 50),
##D               cor=0.05, phi=phi.hat[3], nu=nu.hat[3])$root)
##D 
##D ##Finally, the full model again but this time update the Beta
##D ##using MH.  Using the default of sample.order=0, this sampling
##D ##scheme is simply a single block MH proposal of all model parameters.
##D ##Also, the first run uses the default tuning value for the Beta
##D ##(i.e., chol(vcov(lm(Y~X))). Both chains use a normal prior on Beta;
##D ##however, the first chain allows for a vague prior, where as the
##D ##second is much too strict.
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "Psi"=list(starting=5, tuning=0.5, prior=Psi.prior),
##D        "phi"=list(starting=0.1, tuning=0.5, prior=phi.prior)
##D        )
##D 
##D ##Chain 1: vague prior normal variance of 1/0.001 = 1000
##D beta.prior <- prior(dist="NORMAL", mu=5, precision=0.001)
##D beta.control <- list(update="MH", prior=beta.prior)
##D 
##D Fit.m5.a <-
##D   spGGT(formula=Y.2~1, run.control=run.control,
##D          coords=coords, var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="exponential")
##D 
##D ##Chain 2: restrictive prior normal variance of 1/5 = 0.2
##D ##(i.e., crazy strict)
##D beta.prior <- prior(dist="NORMAL", mu=5, precision=5)
##D beta.control <- list(update="MH", prior=beta.prior, tuning=0.75)
##D 
##D Fit.m5.b <-
##D   spGGT(formula=Y.2~1, run.control=run.control,
##D          coords=coords, var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="exponential")
##D 
##D plot(mcmc.list(Fit.m5.a$p.samples, Fit.m5.b$p.samples))
##D 
##D ##Now just for fun, here is the full model but using the weakly
##D ##informative half-Cauchy for the nugget (Psi) and partial sill (K).
##D 
##D K.prior <- prior(dist="HC", a=50)
##D Psi.prior <- prior(dist="HC", a=50)
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D 
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "Psi"=list(starting=5, sample.order=1, tuning=2, prior=Psi.prior),
##D        "phi"=list(starting=0.1, sample.order=2, tuning=1, prior=phi.prior)
##D        )
##D 
##D beta.control <- list(update="GIBBS", prior=prior(dist="FLAT"))
##D 
##D run.control <- list("n.samples"=3000, "sp.effects"=TRUE)
##D 
##D Fit.m1 <- spGGT(formula=Y.2~1, run.control=run.control,
##D                  coords=coords, var.update.control=var.update.control,
##D                  beta.update.control=beta.control,
##D                  cov.model="exponential")
##D 
##D plot(Fit.m1$p.samples)
##D summary(Fit.m1$p.samples)
##D 
##D #############################
##D ##   Multivariate models
##D #############################
##D 
##D ##Full model with non-spatial cross-covariance
##D ##(Psi) matrix, spatial cross-covariance matrix (K),
##D ##and response specific spatial range parameter (Phi)
##D ##(i.e., a non-separable model).
##D 
##D ##These chains really need to be run for several thousand
##D ##samples (e.g., 10,000). Also note that it is much easier to maintain
##D ##a healthy acceptance rate for each parameter if they
##D ##are updated individually using the sample.order
##D ##directive, as shown in the second example below.
##D K.prior <- prior(dist="IWISH", df=2, S=diag(c(3, 6)))
##D Psi.prior <- prior(dist="IWISH", df=2, S=diag(c(7, 5)))
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D 
##D ##The matrix starting values for K and Psi are actually
##D ##passed as the sqrt of the desired starting values.
##D ##This is only done if K or Psi are full cross-covariance
##D ##matrices and serve to insure that the true starting value
##D ##matrix is positive definite.
##D K.starting <- matrix(c(2,-3, 0, 1), 2, 2)
##D Psi.starting <- diag(c(3, 2))
##D 
##D var.update.control <-
##D   list("K"=list(starting=K.starting, tuning=diag(c(0.08, 0.3, 0.08)),
##D          prior=K.prior),
##D        "Psi"=list(starting=Psi.starting, tuning=diag(c(0.08, 0.3, 0.08)),
##D          prior=Psi.prior),
##D        "phi"=list(starting=0.1, tuning=0.5,
##D          prior=list(phi.prior, phi.prior))
##D        )
##D 
##D beta.control <- list(update="GIBBS", prior=prior(dist="FLAT"))
##D 
##D run.control <- list("n.samples"=2000, "sp.effects"=FALSE)
##D 
##D Fit.mv.m1 <-
##D   spGGT(formula=list(Y.1~1, Y.2~1), run.control=run.control,
##D          coords=coords, var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="exponential")
##D 
##D plot(Fit.mv.m1$p.samples)
##D 
##D ##Calculate the mean K correlation matrix.  Note that since K is
##D ##specified as a full cross-covariance matrix, the posterior
##D ##samples are returned for the lower triangle of the symmetric K,
##D ##in column major order.
##D K <- matrix(0, 2, 2)
##D K[lower.tri(K, diag=TRUE)] <-
##D   colMeans(Fit.mv.m1$p.samples[,c("K_1","K_2","K_3")])
##D K[upper.tri(K, diag=FALSE)] <- t(K)[upper.tri(K, diag=FALSE)]
##D print(cov2cor(K))
##D 
##D ##Now using sample order.  Note this takes ~3 times as long
##D ##to run, but provides much finer control of acceptance rates.
##D var.update.control <-
##D   list("K"=list(sample.order=0, starting=K.starting,
##D          tuning=diag(c(0.3, 0.5, 0.2)), prior=K.prior),
##D        "Psi"=list(sample.order=1, starting=Psi.starting,
##D          tuning=diag(c(0.2, 0.5, 0.2)), prior=Psi.prior),
##D        "phi"=list(sample.order=2, starting=0.1,
##D          tuning=0.5, prior=list(phi.prior, phi.prior))
##D        )
##D 
##D run.control <- list("n.samples"=2000, "sp.effects"=TRUE)
##D 
##D Fit.mv.m2 <-
##D   spGGT(formula=list(Y.1~1, Y.2~1), run.control=run.control,
##D          coords=coords, var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="exponential")
##D 
##D plot(Fit.mv.m2$p.samples)
##D 
##D ##Now again check out the random spatial effects.  Recall,
##D ##these are stored in the sp.effects matrix, which is organized
##D ##as m consecutive rows for each point.
##D rand.effect.Y.1 <-
##D   Fit.mv.m2$sp.effects[seq(1, nrow(Fit.mv.m2$sp.effects), 2),]
##D 
##D rand.effect.Y.2 <-
##D   Fit.mv.m2$sp.effects[seq(2, nrow(Fit.mv.m2$sp.effects), 2),]
##D 
##D rand.effect.Y.1.surf <-
##D   mba.surf(cbind(coords, rowMeans(rand.effect.Y.1)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D rand.effect.Y.2.surf <-
##D   mba.surf(cbind(coords, rowMeans(rand.effect.Y.2)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D ##The estimated negative value of off-diagonal element in K is
##D ##clearly seen in the map of random spatial effects.
##D par(mfrow=c(1,2))
##D image(rand.effect.Y.1.surf, xaxs="r", yaxs="r",
##D       main="Y.1 random spatial effects")
##D contour(rand.effect.Y.1.surf, add=TRUE)
##D image(rand.effect.Y.2.surf, xaxs="r", yaxs="r",
##D       main="Y.2 random spatial effects")
##D contour(rand.effect.Y.2.surf, add=TRUE)
##D 
##D ##Now a simpler model.  Specifically, the full model shows that
##D ##there is negligible non-spatial cross-covariance, therefore,
##D ##Psi might just be modeled with a diagonal covariance matrix.
##D ##This is actually the model which gave rise to this synthetic
##D ##data set.
##D K.prior <- prior(dist="IWISH", df=2, S=diag(c(3, 6)))
##D Psi.prior <- prior(dist="IG", shape=2, scale=5)
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D 
##D var.update.control <-
##D   list("K"=list(sample.order=0, starting=K.starting,
##D          tuning=diag(c(0.3, 0.5, 0.2)), prior=K.prior),
##D        "Psi"=list(sample.order=1, starting=5,
##D          tuning=0.5, prior=list(Psi.prior, Psi.prior)),
##D        "phi"=list(sample.order=2, starting=0.1,
##D          tuning=0.5, prior=list(phi.prior, phi.prior))
##D        )
##D 
##D run.control <- list("n.samples"=2000)
##D 
##D Fit.mv.m3 <-
##D   spGGT(formula=list(Y.1~1, Y.2~1), run.control=run.control,
##D          coords=coords, var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="exponential")
##D 
##D plot(Fit.mv.m3$p.samples)
##D summary(Fit.mv.m3$p.samples)
##D 
##D ##Finally, a really simple model which we know makes no sense
##D ##for this data set.  Specifically, a single variance parameter
##D ##for the diagonal elements of K and Psi and common spatial range,
##D ##phi.  Also, we use a spherical spatial covariance function, just
##D ##because we can.
##D K.prior <- prior(dist="IG", shape=2, scale=5)
##D Psi.prior <- prior(dist="IG", shape=2, scale=5)
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D 
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "Psi"=list(starting=5, tuning=0.5, prior=Psi.prior),
##D        "phi"=list(starting=0.1, tuning=0.5, prior=phi.prior)
##D        )
##D 
##D Fit.mv.m4 <-
##D   spGGT(formula=list(Y.1~1, Y.2~1), run.control=run.control,
##D          coords=coords, var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="spherical")
##D 
##D plot(Fit.mv.m4$p.samples)
##D 
## End(Not run)



